@@ -40,15 +40,16 @@
 #include "disclosure-widget.h"
 #include "menu-fentry.h"
 #include "menu.h"
-#include "multiscreen-stuff.h"
-#include "panel-main.h"
 #include "panel-util.h"
-#include "panel-gconf.h"
+#include "panel-profile.h"
 #include "quick-desktop-reader.h"
 #include "nothing.h"
 #include "egg-screen-exec.h"
 #include "egg-screen-url.h"
 #include "panel-stock-icons.h"
+#include "panel-multiscreen.h"
+#include "panel-a11y.h"
+#include "panel-globals.h"
 
 enum {
 	COLUMN_ICON,
@@ -67,9 +68,6 @@ typedef enum {
 #define ENABLE_LIST_DEFAULT TRUE
 #define SHOW_LIST_DEFAULT FALSE
 
-extern GtkTooltips *panel_tooltips;
-extern gboolean no_run_box;
-
 static GtkWidget *run_dialog = NULL;
 static GSList *add_icon_paths = NULL;
 static guint add_icon_idle_id = 0;
@@ -585,7 +583,6 @@ browse_ok (GtkWidget *widget, GtkFileSelection *fsel)
 	fname = gtk_file_selection_get_filename (fsel);
 	if (fname != NULL) {
 		append_file (entry, fname);
-		gtk_widget_grab_focus (entry);
 	}
 	
 	gtk_widget_destroy (GTK_WIDGET (fsel));
@@ -664,21 +661,15 @@ entry_event (GtkEntry * entry, GdkEventKey * event, gpointer data)
 static void
 sync_entry_to_list (GtkWidget *dialog)
 {
-        gboolean blocked;
-	gboolean enable_program_list;
 	GtkWidget *entry;
-	const char *key;
+        gboolean   blocked;
 
         blocked = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (dialog),
 						      "sync_entry_to_list_blocked"));
         if (blocked)
                 return;
 
-	key = panel_gconf_general_key
-		(panel_gconf_get_profile (), "enable_program_list"),
-	enable_program_list = panel_gconf_get_bool (key, ENABLE_LIST_DEFAULT);
-	
-	if (enable_program_list) {
+	if (panel_profile_get_enable_program_list ()) {
 	        unset_selected (dialog);
 
 		entry = g_object_get_data (G_OBJECT (dialog), "entry");
@@ -818,12 +809,7 @@ static void
 toggle_contents (GtkWidget *disclosure,
                  GtkWidget *dialog)
 {
-	const char *key;
-
-	key = panel_gconf_general_key
-		(panel_gconf_get_profile (), "show_program_list"),
-
-	panel_gconf_set_bool (key, GTK_TOGGLE_BUTTON (disclosure)->active);
+	panel_profile_set_show_program_list (GTK_TOGGLE_BUTTON (disclosure)->active);
 
 	/* FIXME: we need to listen on this key! */
         update_contents (dialog);
@@ -833,18 +819,12 @@ static GtkWidget*
 create_disclosure_widget (void)
 {
         GtkWidget *disclosure;
-        gboolean show_program_list;
-	const char *key;
 
         disclosure = cddb_disclosure_new (_("Known Applications"),
 					  _("Known Applications"));
 
-	key = panel_gconf_general_key
-		(panel_gconf_get_profile (), "show_program_list"),
-	show_program_list = panel_gconf_get_bool (key, SHOW_LIST_DEFAULT);
-
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (disclosure),
-				      show_program_list);
+				      panel_profile_get_show_program_list ());
 		
 	g_object_set_data (G_OBJECT (run_dialog), "disclosure", disclosure);
 	  
@@ -936,8 +916,6 @@ create_simple_contents (GdkScreen *screen)
 	GtkWidget *vbox2;
 	GtkWidget *hbox2;
         GtkWidget *w;
-	const char *key;
-	gboolean enable_program_list;
 	int width_request;
 	static GtkTargetEntry drop_types[] = { { "text/uri-list", 0, 0 } };
         
@@ -964,7 +942,7 @@ create_simple_contents (GdkScreen *screen)
 			    TRUE, TRUE, GNOME_PAD_SMALL);
 
         /* 1/4 the width of the first monitor should be a good value */
-	width_request = multiscreen_width (gdk_screen_get_number (screen), 0) / 4;
+	width_request = panel_multiscreen_width (screen, 0) / 4;
 	g_object_set (G_OBJECT (gentry), "width_request", width_request, NULL);
 
         entry = gnome_entry_gtk_entry (GNOME_ENTRY (gentry));
@@ -1014,12 +992,8 @@ create_simple_contents (GdkScreen *screen)
 	gtk_box_pack_start (GTK_BOX (hbox2), w,
 			    FALSE, FALSE, 0);
 
-	key = panel_gconf_general_key
-		(panel_gconf_get_profile (), "enable_program_list"),
-	enable_program_list = panel_gconf_get_bool (key, ENABLE_LIST_DEFAULT);
-
 	/* only create disclosure widget if really needed */
-	if (enable_program_list) {
+	if (panel_profile_get_enable_program_list ()) {
 	        w = create_disclosure_widget ();
 		gtk_box_pack_start (GTK_BOX (vbox), w,
 				    FALSE, FALSE, GNOME_PAD_SMALL);
@@ -1185,11 +1159,10 @@ find_icon_timeout (gpointer data)
 		if (icon != NULL) {
 			pixbuf = gdk_pixbuf_new_from_file (icon, NULL);
 			g_free (icon);
-			if (pixbuf != NULL) {
+			if (pixbuf != NULL)
 				gtk_image_set_from_pixbuf (GTK_IMAGE (pixmap),
 							   pixbuf);
-				g_object_unref (pixbuf);
-			}
+			g_object_unref (pixbuf);
 		}
 		g_free (found_icon);
 	}
@@ -1523,9 +1496,9 @@ create_program_list_contents (void)
 
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (list), FALSE);
 
-	panel_set_atk_name_desc (list,
-				 _("List of known applications"),
-				 _("Choose an application to run from the list"));
+	panel_a11y_set_atk_name_desc (list,
+				      _("List of known applications"),
+				      _("Choose an application to run from the list"));
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (list));
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
@@ -1567,59 +1540,40 @@ create_program_list_contents (void)
 static void
 update_contents (GtkWidget *dialog)
 {
-	GtkWidget *program_list_box = NULL;
-	GtkWidget *entry;           
-	gboolean show_program_list;
-	const char *key;
+        GtkWidget *program_list_box = NULL;
 
-	key = panel_gconf_general_key
-		(panel_gconf_get_profile (), "show_program_list"),
-	show_program_list = panel_gconf_get_bool (key, SHOW_LIST_DEFAULT);
-        
-	if (show_program_list) {
-		program_list_box = g_object_get_data (G_OBJECT (dialog),
-						      "program_list_box");
+        if (panel_profile_get_show_program_list ()) {
+                program_list_box = g_object_get_data (G_OBJECT (dialog), "program_list_box");
 
-		if (program_list_box && program_list_box->parent == NULL) {
+                if (program_list_box && program_list_box->parent == NULL) {
 			GtkWidget *list;
-			GtkTreeSelection *selection;
-			GtkTreeModel *model;
-			GtkTreeIter iter;
-
+			
 			gtk_window_resize (GTK_WINDOW (dialog), 100, 300);
 			gtk_window_set_resizable (GTK_WINDOW (dialog), TRUE);
+			
+                        gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
+                                            program_list_box,
+                                            TRUE, TRUE, GNOME_PAD_SMALL);
+                
 
-			gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
-					    program_list_box,
-					    TRUE, TRUE, GNOME_PAD_SMALL);
-
-			gtk_widget_show_all (GTK_WIDGET (GTK_DIALOG (dialog)->vbox));
+                        gtk_widget_show_all (GTK_WIDGET (GTK_DIALOG (dialog)->vbox));
 
 			list = g_object_get_data (G_OBJECT (dialog), "program_list");
-			selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (list));
-
-			if (gtk_tree_view_get_model (GTK_TREE_VIEW (list)) &&
-			    !gtk_tree_selection_get_selected (selection,&model, &iter)) {
-			
-				entry = g_object_get_data (G_OBJECT (dialog), "entry");
-				gtk_widget_grab_focus (entry);
-			} else
-				gtk_widget_grab_focus (list);
+			gtk_widget_grab_focus (list);
 		}
-	} else {    
+        } else {    
+		GtkWidget *entry;           
 
-		program_list_box = g_object_get_data (G_OBJECT (dialog),
-						      "program_list_box");
+                program_list_box = g_object_get_data (G_OBJECT (dialog), "program_list_box");
                 
-		if (program_list_box && program_list_box->parent != NULL) {
-			gtk_container_remove (GTK_CONTAINER (program_list_box->parent),
-					      program_list_box);
+                if (program_list_box && program_list_box->parent != NULL) {
+                        gtk_container_remove (GTK_CONTAINER (program_list_box->parent), program_list_box);
 			gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 		}
 
 		entry = g_object_get_data (G_OBJECT (dialog), "entry");
-		gtk_widget_grab_focus (entry);
-	}
+                gtk_widget_grab_focus (entry);
+        }
 }
 
 static void
@@ -1646,9 +1600,7 @@ run_dialog_destroyed (GtkWidget *widget)
 void
 show_run_dialog (GdkScreen *screen)
 {
-        gboolean  enable_program_list;
 	GtkWidget *w;
-	const char *key;
 	char      *run_icon;
 
 	if (no_run_box)
@@ -1699,11 +1651,7 @@ show_run_dialog (GdkScreen *screen)
 
         create_simple_contents (screen);
 	
-	key = panel_gconf_general_key
-		(panel_gconf_get_profile (), "enable_program_list"),
-	enable_program_list = panel_gconf_get_bool (key, ENABLE_LIST_DEFAULT);
-
-	if (enable_program_list) {
+	if (panel_profile_get_enable_program_list ()) {
 		create_program_list_contents ();
 	        update_contents (run_dialog);        
 		
@@ -1713,13 +1661,11 @@ show_run_dialog (GdkScreen *screen)
 			g_idle_add_full (G_PRIORITY_LOW, add_items_idle,
 			 		 w, NULL);
 
-	} 
-
-	/* None of the apps in the list are selected when the dialog is launched.
-	   Hence entry can be safely grabbed
-	*/
-	w = g_object_get_data (G_OBJECT (run_dialog), "entry");
-	gtk_widget_grab_focus (w);
+		gtk_widget_grab_focus (w);
+	} else {
+		w = g_object_get_data (G_OBJECT (run_dialog), "entry");
+		gtk_widget_grab_focus (w);
+	}
 	
 	gtk_widget_show_all (run_dialog);
 }
